<div id="map" style="width: @width@px; height: @height@px;"></div>
<link rel="stylesheet" href="/intranet-leaflet/resources/Control.Geocoder.css" />
<script src="/intranet-leaflet/resources/Control.Geocoder.js"></script>

<script type='text/javascript' <if @::__csp_nonce@ not nil>nonce="@::__csp_nonce;literal@"</if>>

// Create a map
var map = L.map('map').setView([@zoom_lat@, @zoom_lng@], @zoom_zoom@);
L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {
    attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
}).addTo(map);


var iconUrls = {
    blue: "/intranet-leaflet/resources/marker-0067a3.png",
    light_blue: "/intranet-leaflet/resources/marker-38aadd.png",
    blue_grey: "/intranet-leaflet/resources/marker-436978.png",
    purple: "/intranet-leaflet/resources/marker-5b396b.png",
    green: "/intranet-leaflet/resources/marker-728224.png",
    light_green: "/intranet-leaflet/resources/marker-72b026.png",
    red: "/intranet-leaflet/resources/marker-a23336.png",
    light_red: "/intranet-leaflet/resources/marker-d13d29.png",
    pink: "/intranet-leaflet/resources/marker-d252b9.png",
    orange: "/intranet-leaflet/resources/marker-f69730.png"
};

// Loop through all companies and put a marker for each.
var geocoder = new L.Control.Geocoder.Nominatim({});
var store = @store;noquote@;
var delay = 0.0;
var errorLat = -60.0;
var errorLng = 0.0;
store.each(function(addrModel) {

    // Get basic properties of the customer
    var companyId = addrModel.get('company_id');
    var companyName = addrModel.get('company_name');
    var officeId = addrModel.get('office_id');
    var lat = parseFloat(addrModel.get('geo_lat'));
    var lng = parseFloat(addrModel.get('geo_lng'));
    var needsUpdate = addrModel.get('needs_update_p');
    var line1 = addrModel.get('line1');
    var line2 = addrModel.get('line2');
    var zip = addrModel.get('zip_code');
    var city = addrModel.get('city');
    var state = addrModel.get('state');
    var country = addrModel.get('country');

    var invoiceAmount = addrModel.get('invoice_amount');
    var color = addrModel.get('color');

    // Address line: May consit of one or two "lines"
    var line = ""
    if (!!line1 && "" != line1) line = line1;
    if (!!line2 && !! line1 && "" != line1 && "" != line2) line = line1+", "+line2;
    
    var zip_city = "";
    if (!!city) zip_city = city;
    if (!!zip && !!city) zip_city = zip+" "+city;   

    // Create a pretty content for the pop-up
    var companyUrl = "/intranet/companies/view?company_id="+companyId;
    var popup = "<b><a href='"+companyUrl+"' target=_>"+companyName+"</a></b><br>";
    if (!!line1 && "" != line1) popup = popup+line1+"<br>";
    if (!!line2 && "" != line2) popup = popup+line2+"<br>";
    if ("" != zip_city) popup = popup+zip_city+"<br>";
    if (!!state && "" != state) popup = popup+state+"<br>";
    if (!!country && "" != country) popup = popup+country+"<br>";
    popup = popup+'Invoiced: '+invoiceAmount+"<br>";

    // Determine the size of the marker icon to be shown:
    // revenue < 1000: f=0.3, revenue > 40000: f=1.0, everything else in the middle
    var sq = Math.sqrt(invoiceAmount); 
    // if (sq > 300) sq = 300.0;
    var f = 0.2 + 0.5 * sq/300;
    var iconUrl = iconUrls[color];
    var icon = L.icon({
        iconUrl: iconUrl,
        iconSize: [25*f, 41*f],
        iconAnchor: [12*f, 41*f],
        popupAnchor: [1, -34*f],
        shadowUrl: '/intranet-leaflet/resources/marker-shadow.png',
        shadowSize: [41*f, 41*f],
        shadowAnchor: [12*f, 41*f]
    });

    // Geo-position was calculated before, so just add the popup
    if (!!lat && !!lng && "0" == needsUpdate) {
        var marker = L.marker([lat, lng], {icon: icon}).addTo(map).bindPopup(popup);
        return;                   // No need to continue with the rest of the function
    };
    
    // No geo position, or needs an update: Build a query from the known address pieces
    var queries = [];
    // if (!!country) query = country;       // don't allow country only, because of collisions
    if (!!city && !country) queries.push(city);
    if (!!state && !!country) queries.push(state+", "+country);
    if (!!city && !!country) queries.push(city+", "+country);
    if (!!zip && !!city && !!country) queries.push(zip+" "+city+", "+country);
    if (!!city && !!state && !!country) queries.push(city+", "+state+", "+country);
    if (!!zip && !!city && !!state && !!country) queries.push(zip+" "+city+", "+state+", "+country);
    if ("" != line && !!zip && !!city && !!state && !!country) queries.push(line+", "+zip+" "+city+", "+state+", "+country);

    if (queries.length == 0) {
        // No city + state, so draw below the equator.
        popup = popup + "Empty address: No city + country available.<br>";
        var marker = L.marker([errorLat+5.0, errorLng], {icon: icon}).addTo(map).bindPopup(popup);
        errorLng = errorLng + 7.77;
        if (errorLng > 180) errorLng = errorLng - 360;
        return;
    }

    while (query = queries.pop()) {

	var orgQuery = ""+query;
	window.setTimeout(geocoder.geocode(query, function(results) {
		    if (0 == results.length) {
			// No results found from geocoding service
			// alert('No result found for query='+orgQuery);
			popup = popup + "Geocoder did not find address: "+orgQuery+"<br>";
			var marker = L.marker([errorLat, errorLng], {icon: icon}).addTo(map).bindPopup(popup);
			errorLng = errorLng + 7.77;
			if (errorLng > 180) errorLng = errorLng - 360;
			return;
		    }

		    // center of the result
		    var first = results[0];
		    var centerLat = first.center.lat;
		    var centerLng = first.center.lng;

		    switch ("@random_within_bbox_p@") {
		    case "1":
			// Move the position within Bounding Box, in order to avoid 
			// "collisions" with two objects in the same country or city
			var west = first.bbox._southWest.lng;
			var east = first.bbox._northEast.lng;
			var north = first.bbox._northEast.lat;
			var south = first.bbox._southWest.lat;
            
			// Move the position randomly within the bounding box
			// in order to handle two customers with the same city+country
			lng = centerLng + (west-centerLng)*Math.random() + (east-centerLng)*Math.random();
			lat = centerLat + (north-centerLat)*Math.random() + (south-centerLat)*Math.random();
	    
			// Show bounding box for random position in bbox
			var polygon = L.polygon([
						 [north, west],
						 [south, west],
						 [south, east],
						 [north, east]
						 ]).addTo(map);
			break;
		    case "2":
			// Add a random +/- 5km to lat+lng. 1 degree lat/lng are ~11km
			var lat = centerLat + (Math.random()-0.5) * 0.1;
			var lng = centerLng + (Math.random()-0.5) * 0.1;
			break;
		    default:
			var lat = centerLat;
			var lng = centerLng;
		    }

		    // Set the marker at the right position
		    L.marker([lat, lng], {icon: icon}).addTo(map).bindPopup(popup);

		    // Write the lat/lng into the database for next call
		    Ext.Ajax.request({
			    url: '/intranet-leaflet/update-office',
				params: { office_id: officeId, lat: lat, lng: lng },
				method: 'GET'
			});
        
		}), delay);
	
	// The geocode service will stop responding if we ask too fast.
	delay = delay + 5000.0;
	
    }

});

</script>

