<if "" eq @message@>
<!-- Show the map only if the message was empty... -->

<div id="map" style="width: @width@px; height: @height@px;"></div>

<script>
var calcStyle = function(hits, num, sum, max) {

    // Discard all countries with less then the average of hits
    // For all other countries scale the color from blue to red.
    
    var avg = sum / num;
    if (hits < avg) return null;
    
    var lnHits = Math.sqrt(hits);
    var lnAvg = Math.sqrt(avg);
    var lnMax = Math.sqrt(max);
    if (Number.isNaN(lnHits)) return null;
    var maxLnHits = lnMax - lnAvg;

    var blue = [0,0,255];
    var red = [255,0,0]

    // normalized 0..0.9999 weight from lnHits range of 0 to ~16
    lnHits = lnHits - lnAvg;
    if (lnHits > maxLnHits) lnHits = maxLnHits;
    if (lnHits < 0.0) lnHits = 0.0;
    var weight = lnHits / maxLnHits;
    var w2 = weight;
    var w1 = 1.0 - w2;
    var col = [
        Math.floor(w1*blue[0] + w2*red[0]),
        Math.floor(w1*blue[1] + w2*red[1]),
        Math.floor(w1*blue[2] + w2*red[2])
    ];

    var colHex = "";
    colHex = colHex + ("0"+col[0].toString(16)).substr(-2);
    colHex = colHex + ("0"+col[1].toString(16)).substr(-2);
    colHex = colHex + ("0"+col[2].toString(16)).substr(-2);

    var styleWeight = 0.5;
    if (lnHits < 3.0) styleWeight = 0.0;
    var styleOpacity = weight;
    if (lnHits < 1.0) styleOpacity = 0.0;

    var myStyle = {
        "color": "#"+colHex,
        "weight": styleWeight,
        "fillOpacity": styleOpacity
    };
    return myStyle;
};


// Create the actual map
var map = L.map('map').setView([@zoom_lat@, @zoom_lng@], @zoom_zoom@);
L.tileLayer('http://{s}.tile.osm.org/{z}/{x}/{y}.png', {
    attribution: '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
}).addTo(map);

// Table of IPs per country
var store = @store;noquote@;

// Load the 23MB list of polygons per country.
// Content is a "features" array with entries for 254 countries.
// Each country has the structure:
// {type: "Feature", properties: {ADMIN: "<country name>", ISO_A3: "GER", ISO_A2: "DE"}, geometry: {...}}
console.log('Loading countries.geo.json');
var featureHash = {};
var statNum = 255;	// fixed 255 countries
var statSum = 0;
var statMax = 0;

var xhr = new XMLHttpRequest();
xhr.open('GET', "/intranet-leaflet/countries.geo.json", true);
xhr.responseType = 'json';
xhr.onload = function() {
    var status = xhr.status;
    if (status === 200) {
        var features = xhr.response.features;
        console.log('Writing countries.geo.json into featureHash');
        for (var i = 0; i < features.length; i++) {
            var countryFeature = features[i];
            var props = countryFeature.properties;
            var iso_a2 = props.ISO_A2.toLowerCase();
            featureHash[iso_a2] = countryFeature;
        }
    } else {
        alert('Leaflet World Countries: Unable to load /intranet-leaflet/countries.geo.json. Status='+status);
    }

    // Aggregate values from Store into the featureHash
    console.log('Aggregating data from countries.geo.json');
    store.each(function(record) {
        var country_code_a2 = record.get('country_code');
        if ("string" !== typeof country_code_a2) return;

	switch (country_code_a2) {
	    case "uk": country_code_a2 = "gb"; break		// UK -> GB
	    case "re": country_code_a2 = "fr"; break		// Reunión is French territory
	    case "yt": country_code_a2 = "fr"; break		// Mayotte is French territory
	}

        var countryFeature = featureHash[country_code_a2];
        if (!countryFeature) {
            console.log('countryFeate for country='+country_code_a2+' not found');
            return;
        }

        var props = countryFeature.properties;
        var hits = parseFloat(record.get('cnt'));
        if (!props.count) props.count = 0.0;
        props.count = props.count + hits;
        statSum = statSum + hits;
        if (props.count > statMax) statMax = props.count;
    });

    // Render
    console.log('Rendering data from countries.geo.json');
    for (var a2 in featureHash) {
        //if (Object.prototype.hasOwnProperty.call(a2, featureHash)) {
            
            var countryFeature = featureHash[a2];
            var props = countryFeature.properties;
            var hits = props.count;
            var iso_a2 = props.ISO_A2;
            var geometry = countryFeature.geometry;
            var country = {"type":"FeatureCollection","features":[countryFeature]};
            
            var myStyle = calcStyle(hits, statNum, statSum, statMax);
            if (!myStyle) continue;
            L.geoJson(country, {style: myStyle}).addTo(map);	      // Add geojson snippet to map
        //}
    }

};
xhr.send();

</script>

    <!-- Show up to here if the message was empty... -->
    </if>
    <else>
    <!-- The message was not empty, so just show the message... -->
    @message;noquote@

    <br>
    switch_string: @switch_string@<br>
    @lock@: @lock_exists_p@<br>
    @aggr@: @aggr_exists_p@<br>

</else>
